@@ -11,18 +11,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import io
 import sys
-
-import mock
-from oslo_utils import encodeutils
-import six
-from six.moves.urllib import parse
+from unittest import mock
+from urllib import parse
 
 from novaclient import base
 from novaclient import exceptions
 from novaclient.tests.unit import fakes
 from novaclient.tests.unit import utils as test_utils
 from novaclient import utils
+from novaclient.v2 import servers
 
 UUID = '8e8ec658-c7b0-4243-bdf8-6f7f2952c0d0'
 
@@ -176,7 +175,7 @@ class _FakeResult(object):
 
 
 class PrintResultTestCase(test_utils.TestCase):
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_dict(self):
         dict = {'key': 'value'}
         utils.print_dict(dict)
@@ -187,7 +186,7 @@ class PrintResultTestCase(test_utils.TestCase):
                          '+----------+-------+\n',
                          sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_dict_wrap(self):
         dict = {'key1': 'not wrapped',
                 'key2': 'this will be wrapped'}
@@ -201,7 +200,7 @@ class PrintResultTestCase(test_utils.TestCase):
                          '+----------+--------------+\n',
                          sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_list_sort_by_str(self):
         objs = [_FakeResult("k1", 1),
                 _FakeResult("k3", 2),
@@ -218,7 +217,7 @@ class PrintResultTestCase(test_utils.TestCase):
                          '+------+-------+\n',
                          sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_list_sort_by_integer(self):
         objs = [_FakeResult("k1", 1),
                 _FakeResult("k3", 2),
@@ -235,14 +234,11 @@ class PrintResultTestCase(test_utils.TestCase):
                          '+------+-------+\n',
                          sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_unicode_list(self):
-        objs = [_FakeResult("k", u'\u2026')]
+        objs = [_FakeResult("k", '\u2026')]
         utils.print_list(objs, ["Name", "Value"])
-        if six.PY3:
-            s = u'\u2026'
-        else:
-            s = encodeutils.safe_encode(u'\u2026')
+        s = '\u2026'
         self.assertEqual('+------+-------+\n'
                          '| Name | Value |\n'
                          '+------+-------+\n'
@@ -251,7 +247,7 @@ class PrintResultTestCase(test_utils.TestCase):
                          sys.stdout.getvalue())
 
     # without sorting
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_list_sort_by_none(self):
         objs = [_FakeResult("k1", 1),
                 _FakeResult("k3", 3),
@@ -268,7 +264,7 @@ class PrintResultTestCase(test_utils.TestCase):
                          '+------+-------+\n',
                          sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_dict_dictionary(self):
         dict = {'k': {'foo': 'bar'}}
         utils.print_dict(dict)
@@ -279,7 +275,7 @@ class PrintResultTestCase(test_utils.TestCase):
                          '+----------+----------------+\n',
                          sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_dict_list_dictionary(self):
         dict = {'k': [{'foo': 'bar'}]}
         utils.print_dict(dict)
@@ -290,7 +286,7 @@ class PrintResultTestCase(test_utils.TestCase):
                          '+----------+------------------+\n',
                          sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_dict_list(self):
         dict = {'k': ['foo', 'bar']}
         utils.print_dict(dict)
@@ -301,7 +297,7 @@ class PrintResultTestCase(test_utils.TestCase):
                          '+----------+----------------+\n',
                          sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_large_dict_list(self):
         dict = {'k': ['foo1', 'bar1', 'foo2', 'bar2',
                       'foo3', 'bar3', 'foo4', 'bar4']}
@@ -315,14 +311,11 @@ class PrintResultTestCase(test_utils.TestCase):
             '+----------+------------------------------------------+\n',
             sys.stdout.getvalue())
 
-    @mock.patch('sys.stdout', six.StringIO())
+    @mock.patch('sys.stdout', io.StringIO())
     def test_print_unicode_dict(self):
-        dict = {'k': u'\u2026'}
+        dict = {'k': '\u2026'}
         utils.print_dict(dict)
-        if six.PY3:
-            s = u'\u2026'
-        else:
-            s = encodeutils.safe_encode(u'\u2026')
+        s = '\u2026'
         self.assertEqual('+----------+-------+\n'
                          '| Property | Value |\n'
                          '+----------+-------+\n'
@@ -402,6 +395,30 @@ class DoActionOnManyTestCase(test_utils.TestCase):
     def test_do_action_on_many_last_fails(self):
         self._test_do_action_on_many([None, Exception()], fail=True)
 
+    @mock.patch('sys.stdout', new_callable=io.StringIO)
+    def _test_do_action_on_many_resource_string(
+            self, resource, expected_string, mock_stdout):
+        utils.do_action_on_many(mock.Mock(), [resource], 'success with %s',
+                                'error')
+        self.assertIn('success with %s' % expected_string,
+                      mock_stdout.getvalue())
+
+    def test_do_action_on_many_resource_string_with_str(self):
+        self._test_do_action_on_many_resource_string('resource1', 'resource1')
+
+    def test_do_action_on_many_resource_string_with_human_id(self):
+        resource = servers.Server(None, {'name': 'resource1'})
+        self._test_do_action_on_many_resource_string(resource, 'resource1')
+
+    def test_do_action_on_many_resource_string_with_id(self):
+        resource = servers.Server(None, {'id': UUID})
+        self._test_do_action_on_many_resource_string(resource, UUID)
+
+    def test_do_action_on_many_resource_string_with_id_and_human_id(self):
+        resource = servers.Server(None, {'name': 'resource1', 'id': UUID})
+        self._test_do_action_on_many_resource_string(resource,
+                                                     'resource1 (%s)' % UUID)
+
 
 class RecordTimeTestCase(test_utils.TestCase):
 
@@ -427,9 +444,8 @@ class PrepareQueryStringTestCase(test_utils.TestCase):
     def setUp(self):
         super(PrepareQueryStringTestCase, self).setUp()
         self.ustr = b'?\xd0\xbf=1&\xd1\x80=2'
-        if six.PY3:
-            # in py3 real unicode symbols will be urlencoded
-            self.ustr = self.ustr.decode('utf8')
+        # in py3 real unicode symbols will be urlencoded
+        self.ustr = self.ustr.decode('utf8')
         self.cases = (
             ({}, ''),
             (None, ''),
